/****************************************************************************
**
** This file is part of PLCEdit, an open-source cross-platform editor 
** for PLC source files (e.g. from Moeller, CoDeSys and Siemens).
** Copyright (C) 2005-2009  M. Rehfeldt
**
** This software uses classes of Trolltech Qt toolkit and is freeware. 
** This file may be used under the terms of the GNU General Public License 
** version 2.0 or (at your option) any later version as published by the 
** Free Software Foundation and appearing in the file LICENSE.GPL included 
** in the packaging of this file. 
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Contact e-mail: M. Rehfeldt <info@plcedit.org>
** Program URL   : http://www.plcedit.org
**
****************************************************************************/

#include <QtGui>
#include <QtCore>


#include "importexport.h"
#include "pagectrl.h"
#include "prototype.h"
#include "pathextdialog.h"


ImportExport::ImportExport(PageCtrl *pageCtrl, Prototype *prototype) 
  : m_PageCtrl(pageCtrl), m_Prototype(prototype) //m_PageCtrl as member VAR of PrefDialog
{ //! creating function
  //! \param PageCtrl class, Prototype class

  progressBar = new QProgressBar(0); //create progressbar
  retranslateUi();					 //create texts
}


ImportExport::~ImportExport()
{ //! virtual destructor

  delete progressBar; //delete progressbar
}


QString ImportExport::createKMHeader(QString Str)
{ //! revise Moeller header from string input
	
  QStringList list1;
  QString tmpStr, dayName, monthName, timeStamp;
  QDate date;
  QTime time;

  //get day string
  switch ( date.currentDate().dayOfWeek() )
  {
  	case 1 : { dayName = "Mon"; break; } //monday
    case 2 : { dayName = "Tue"; break; } //Tuesday
    case 3 : { dayName = "Wed"; break; } //Wednesday
    case 4 : { dayName = "Thu"; break; } //Thursday
    case 5 : { dayName = "Fri"; break; } //Friday
    case 6 : { dayName = "Sat"; break; } //Saturday
    case 7 : { dayName = "Sun"; break; } //Sunday
  }
  //get month string
  switch ( date.currentDate().month() )
  {
  	case 1 : { monthName = "Jan"; break; } //January
    case 2 : { monthName = "Feb"; break; } //February
    case 3 : { monthName = "Mar"; break; } //March
    case 4 : { monthName = "Apr"; break; } //April
    case 5 : { monthName = "May"; break; } //May
    case 6 : { monthName = "Jun"; break; } //June
    case 7 : { monthName = "Jul"; break; } //July
    case 8 : { monthName = "Aug"; break; } //August
    case 9 : { monthName = "Sep"; break; } //September
    case 10: { monthName = "Oct"; break; } //October
    case 11: { monthName = "Nov"; break; } //November
    case 12: { monthName = "Dec"; break; } //December
  }  
  //set timestamp
  timeStamp = "TIMESTAMP = " 
  + dayName + " " + monthName + " " + date.currentDate().toString("dd") 
  + " " + time.currentTime().toString("hh:mm:ss") 
  + " " + date.currentDate().toString("yyyy");     

  
  list1.clear();
  
  if (!Str.isEmpty())
  { //if string is filled replace only cahngeable items
    list1 = Str.split("\n"); 
    for (int i=0; i<list1.size(); i++)
    {
      tmpStr = list1[i];
      if (tmpStr.contains("KM_TEXT"))	
        list1[i] = m_PageCtrl->KM_TEXT;
      if (tmpStr.contains("S40_VERSION"))	
        list1[i] = m_PageCtrl->S40_VERSION;
      if (tmpStr.contains("TIMESTAMP"))	
        list1[i] = timeStamp;
      if (tmpStr.contains("tKisInstrEdi"))
      	if (m_PageCtrl->currentPrgLng() == "IL")
      	  list1[i] = "tKisInstrEdi = kAwl"; //SucoSoft AWL marker
      	else 
      	  list1[i] = "tKisInstrEdi = ST"; //SucoSoft ST marker
      if (tmpStr.contains("tKisSplitMode"))
        if (m_PageCtrl->currentSplitMode() == "v")
          list1[i] = "tKisSplitMode = kVer";
        else 
          list1[i] = "tKisSplitMode = kHor";
      if (tmpStr.contains("m_ulSplitSize"))
        list1[i] = ("m_ulSplitSize = " + QString::number(m_PageCtrl->currentSplitSize())); 
    }
  }
  else
  { //if string was empty set complete new Moeller SucoSoft file header
    list1.append("$BEGIN_KM_HEADER");
    list1.append("<LINES=00004-SIZE=00000078-NEXTPOS=00000000>");
    list1.append(m_PageCtrl->KM_TEXT);
    list1.append(m_PageCtrl->S40_VERSION);
    list1.append("CRC = 34102");
    list1.append(timeStamp);
    list1.append("<LINES=00010-SIZE=0000012B-NEXTPOS=00000000>");
    list1.append("tKisDeclEdi = kTxt");
    if (m_PageCtrl->currentPrgLng() == "IL")
      list1.append("tKisInstrEdi = kAwl"); 	//SucoSoft AWL marker
    else 
      list1.append("tKisInstrEdi = ST"); 	//SucoSoft ST marker
    list1.append("tKisWinArea = kImpl");
    list1.append("m_WndRect = 0#0#650#283#");
    if (m_PageCtrl->currentSplitMode() == "v")
      list1.append("tKisSplitMode = kVer");
    else 
      list1.append("tKisSplitMode = kHor");
    list1.append("tKisKindOfDeclBlock = kDeclVar");
    list1.append("m_ulSplitSize = " + QString::number(m_PageCtrl->currentSplitSize())); 
    list1.append("m_InstrPos = 0/0#0/0#0/0#");
    list1.append("m_DeclPos = 0/0#0/0#");
    list1.append("m_ulGridColWidth = ");
    list1.append("<LINES=00007-SIZE=00000059-NEXTPOS=00000000>");
    list1.append("PROJECT = ");
    list1.append("OEMNAME = ");
    list1.append("OEMDATE = ");
    list1.append("COMPANY = ");
    list1.append("VERSION = ");
    list1.append("SPS_TYPE = ");
    list1.append("POETXTSIZE = 0");
    list1.append("$END_KM_HEADER");
  }
  
  //return list 
  return(list1.join("\n"));
}


QString ImportExport::currentFileName()
{ //! return file name of current page

  return(m_PageCtrl->currentFileName());
}


int ImportExport::currentTabIndex()
{ //! return current page index

  return(m_PageCtrl->currentTabIndex());
}




//=============================================================================
// - - - public SLOTs - - -
void ImportExport::setProgressBar(int files, int minFiles, QString msg)
{ //! set progressbar with number of files, min. files to be activated, message string

  if (files < minFiles)
    return; //return if progressbar is not needed (not enough files)
  
  #ifdef Q_WS_MAC 
    progressBar->setStyle(new QCleanlooksStyle());
  #endif
  
  progressBar->setGeometry((QRect(QDesktopWidget().screenGeometry()).width()-200)/2, (QRect(QDesktopWidget().screenGeometry()).height()-28)/2, 200, 28);
  progressBar->setWindowFlags(Qt::Tool | Qt::WindowStaysOnTopHint);
  progressBar->setAlignment(Qt::AlignHCenter);
  progressBar->reset();
  progressBar->setRange(0, files);
  progressBar->setFormat(msg + "  %v / %m");
  progressBar->show();
}


void ImportExport::retranslateUi()
{ //! retranslate Ui

//messages
  msg_progress_fopen = tr("open file");
  msg_progress_batch = tr("batch progress");
  msg_error_pou_line = tr("in line");
}


//------------------------------------------------------------------------------------
// B A T C H   F U N C T I O N + + + + + + + + + + + + + + + + + + + + + + + + + + + +
//------------------------------------------------------------------------------------
void ImportExport::batchConverter()
{ //! batch converting / file resaving function
  /*! The batch dialog returns the full string of the combobox: */
  /*! 'only copy' */
  /*! 'clean POU (.txt)' */
  /*! 'Moeller POE (.poe)' */
  /*! 'CoDeSys (.EXP)' */
  /*! 'Siemens S7 (.awl / .scl - keep current language)' */
  /*! 'KW-Software (.IL / .ST - keep current language)' */
  /*! If Siemens of KW-Software were detected the called saveAllPages(QString batchPath, int batchExt) function will only get batchExt=4 or batchExt=6 to keep the current programming language by self detection. */
  	
  int  batchExt=0;
  QString batchPath, batchExtStr;

  if (m_PageCtrl->countPages() < 1)
    return; //return if no tab exists
    
  PathExtDialog *batchDialog = new PathExtDialog(0);
  
  batchDialog->setWindowFlags( Qt::Tool);
  batchDialog->presetPath(lastDir);
  if (!batchDialog->exec())
  { delete batchDialog; return; } //delete dialog and return if exec()=0
   
  //set wait cursor
  QApplication::setOverrideCursor(Qt::WaitCursor);
  emit active();
  
  //get path and extension
  batchPath = batchDialog->currentPath();
  batchExtStr = batchDialog->currentExt();
  
  //set path if not exists
  if (!QDir(batchPath).exists());
    QDir(batchPath).mkpath(batchPath);
  
  //get file extension as INT and Str
  if (batchExtStr.contains(".txt", Qt::CaseInsensitive))
    batchExt = 1; 
  if (batchExtStr.contains(".poe", Qt::CaseInsensitive))
    batchExt = 2; 
  if (batchExtStr.contains(".exp", Qt::CaseInsensitive))
    batchExt = 3; 
  if (batchExtStr.contains("Siemens", Qt::CaseInsensitive))
    batchExt = 4; 
  if (batchExtStr.contains("KW-Software", Qt::CaseInsensitive))
    batchExt = 6;  
  if (batchExtStr.contains("only copy", Qt::CaseInsensitive))
    batchExt = 99;  
	    
  saveAllPages(batchPath, batchExt); 	//call save all pages routine
  
  //clean up
  delete batchDialog;					//delete batchDialog 
  
  QApplication::restoreOverrideCursor();//give std. cursor back
  emit ready();							//emit ready signal
}


void ImportExport::saveAllPages(QString batchPath, int batchExt)
{ //! save all pages
  //! \param batchPath contains save path, if empty call PageCtrl->filePathOf(i)
  //! \param batchExt contains the selected file extension, if set to 99 use PageCtrl->fileExtOf(i)
  /*! supported extension values: */
  /*! - 0/1 = .txt */
  /*! - 2 = .poe */
  /*! - 3 = .EXP */
  /*! - 4 = .awl */
  /*! - 5 = .scl */
  /*! - 6 = .IL */
  /*! - 7 = .ST */
  /*! - 99 = PageCtrl->fileExtOf(i) */
      
  bool error=false;
  int Ext=0, i=0;
  QString path, fileName, ExtStr;
 
  if (m_PageCtrl->countPages() < 1)
    return; //return if no tab exists
    
  QStringList *tmpList = new QStringList();
  
  setProgressBar(m_PageCtrl->countPages(), 1, msg_progress_batch); //set progressbar
  	
  for (i=0; i < m_PageCtrl->countPages(); i++)//for1
  { 
    //inc. progressbar
    progressBar->setValue(i); 
    progressBar->update();
    QApplication::processEvents((QEventLoop::AllEvents));
  	//clear tmpList
	tmpList->clear();
	//get current filename
	fileName = m_PageCtrl->fileNameOf(i);
	
	if ((fileName.isEmpty() == false) && (fileName != "untitled"))//if1
	{ //only save file if name is not empty/'untitled'
	  
	  //check if only copying or converting requested (value 99)
	  if (batchExt == 99)
	  {
	  	Ext = m_PageCtrl->fileExtOf(i);
	  }
	  else
	  	Ext = batchExt;
	  //get EXT string
	  switch ( Ext )
      {
  	    case 1 : { ExtStr = ".txt"; break; }
	  	case 2 : { ExtStr = ".poe"; break; }
	  	case 3 : { ExtStr = ".EXP"; break; }
	  	case 4 : { ExtStr = ".awl"; Ext = 4;
	  	 		   if (m_PageCtrl->prgLngOf(i) == "ST") //used if batch extension was choosen
	  	 		   {ExtStr = ".scl"; Ext = 5;}
	  	 		   break;                  }
	  	case 5 : { ExtStr = ".scl"; break; }			//used if 'only copy' was choosen
	  	case 6 : { ExtStr = ".IL" ; Ext = 6;
	  	 		   if (m_PageCtrl->prgLngOf(i) == "ST")  //used if batch extension was choosen
	  	 		   {ExtStr = ".ST"; Ext = 7;}
	  	 		   break;                  }
	  	case 7 : { ExtStr = ".ST";  break; }			//used if 'only copy' was choosen
	  	default: { ExtStr = ".txt"; break; }
	  }
	
	  //call saveprepare fcn. for page 'i'
      SavePrepare(i, fileName, tmpList, Ext); 
    
      //check batch path if empty (is empty if called by prefrences timer)
      if (batchPath.isEmpty())
        path = m_PageCtrl->filePathOf(i); //use page information and original path
      else
      	path = batchPath;				 //use batchPath
      
      //create file
      QFile file(path + fileName + ExtStr);
      if (!file.open(QFile::WriteOnly | QIODevice::Text))//if3
        error = true;
      if (error == false)//if4
      { //create text stream
        QTextStream ts(&file);
        //set text codec
        ts.setCodec(QTextCodec::codecForName("UTF-8"));
        //save tmpList from saveprepare fcn. to file
        ts << tmpList->join("\n");   
      }//if4
    }//if1
  }//for1
  
  progressBar->hide(); //hide progressbar
  
  delete tmpList; 		//delete tmpList	
}




//=============================================================================
// - - - diverse public functions - - - 
//------------------------------------------------------------------------------------
// O P E N   F U N C T I O N S + + + + + + + + + + + + + + + + + + + + + + + + + + + +
//------------------------------------------------------------------------------------
bool ImportExport::OpenPrepare(QStringList files)
{ //! prepare file opening of string list content and call open reinforce function
	
  QFile		  file;
  bool        okOpenReinforce=false, error=false;
  int         i=0, j=0, pouEndIndex=0, listLength=0;
  QString     filePath="", fileName="", ExtStr="";
  QStringList *tmpList = new QStringList();
  QStringList extList;
  extList << ".txt" << ".poe" << ".EXP" << ".awl" << ".scl" << ".IL" << ".ST";

  //clear errorFiles list
  errorFiles.clear();
  //set  inhibit create highlighters flag to prevent slow opening, depending on highlighters creation
  m_PageCtrl->flg_inhibit_createHighlighters = true; 
  //set wait cursor
  QApplication::setOverrideCursor(Qt::WaitCursor);
  emit active();
  
  setProgressBar(files.count(), 3, msg_progress_fopen); //set progressbar
  
  //if files are selected, get each in a new page
  for (i=0; i< files.count(); i++)//for1
  { 
    //inc. progressbar
    progressBar->setValue(i); 
    progressBar->update();
    QApplication::processEvents((QEventLoop::AllEvents));
  	//clear tmplist and reset error
    tmpList->clear();
    error = false; 

    //get stripped file name
    fileName = strippedFileName(files[i]); //strippedName contains already .simplified() 
    //get file extension
    ExtStr = fileName;
    ExtStr = ExtStr.remove(0, ExtStr.lastIndexOf("."));
     
    //check if extension correct
    if (!extList.contains(ExtStr, Qt::CaseInsensitive))
      error = true;
    //set filename with full path information
    file.setFileName(files[i].simplified());
    if (!file.exists())//if1
  	  error = true; //set error flag
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))//if2
  	  error = true; //set error flag
    
    if (error == false)//if3
    { 
      //get file into stream
  	  QTextStream in(&file);
      //get file content into StringList
      while (!file.atEnd())
        tmpList->append(file.readLine()); 
        
      //get maybe included files by file type end index
      listLength = tmpList->count();
      for (j=0; j < listLength; j++)//for2
      {
        m_PageCtrl->newPage(); 							//create new page
        pouEndIndex = nextPOUEndIndexAt(tmpList, j); 	//get next pou end index
        
        if (pouEndIndex > -1) //if4
        {
          okOpenReinforce = OpenReinforce(tmpList, j, pouEndIndex, ExtStr);

          if (okOpenReinforce == true)//if5
          { //get current file path
            filePath = files[i]; 
            m_PageCtrl->setCurrentFilePath(filePath.remove(filePath.lastIndexOf("/")+1, filePath.length())); //set file path
            m_PageCtrl->setPageModifiedOf(m_PageCtrl->currentTabIndex(), false); //set page modified to false
          }
          else 
	      { //pou error handling
	        m_PageCtrl->deletePage(); //delete page
	        errorFiles.append(files[i].simplified() + "<br>" + msg_error_pou_line + ": " + QString::number(j+1)); //get file path and error line
	      }//if5
	    }
  	    
  	    //prepare variables for next loop run
  	    listLength = tmpList->count();
  	    j = pouEndIndex+1;
  	    
  	    if (pouEndIndex < 0)//if6
  	    { //end loop if fileEndIndex was returned as fals value
  	      m_PageCtrl->deletePage(); 
  	      break;
  	    }//if6
 	  }//for2
	}
	else
	{ //fill errorFiles list
	  errorFiles.append(files[i].simplified());
	}//if3
  }//for1
  
  progressBar->hide(); //hide progressbar
  
  //reset inhibit create highlighters flag
  m_PageCtrl->flg_inhibit_createHighlighters = false; 
  if (m_PageCtrl->countPages() > 0)//if8
  { //set List and Tab component visible, needed if no page is already active
    m_PageCtrl->createHighlighters(0);		//call int-SLOT create highlighter, for last opened file
    m_PageCtrl->setFileListVisible(m_PageCtrl->flg_fileListVisible);	//set file list visible 
    m_PageCtrl->setTabCtrlVisible(true);	//set tab ctrl. visible
  }//if8

  //check status of use preset dir flag
  if (usePresetDir == false)//if9
  { //save last dir
    lastDir = files[i-1];					//get last Dir, (=i-1)
    lastDir = lastDir.remove(lastDir.lastIndexOf("/")+1, lastDir.length()); 
  }//if9

  emit modified();						 //emit modified information
  
  //clean up
  delete tmpList;						//delete tmpList
  
  QApplication::restoreOverrideCursor();//give std. cursor back
  emit ready();							//emit ready signal
  
  //return handling
  if (errorFiles.count() < 1)//if10
    return(true);
  else
    return(false);
}


int ImportExport::nextPOUEndIndexAt(QStringList *tmpList, int index)
{ //! return next POU end index of tmpList up from index
	
  bool		end_found=false;
  int       i=0;
  QString   tmpStr="";
  
  //get pos information and lines content
  if (tmpList->count() < 0)//if1
    return(-1);
   
  for (i = index; i < tmpList->count(); i++)//for1
  { //run through tmpList
    tmpStr = tmpList->at(i);//get single line
    tmpStr.simplified();
    //get FileEnd position
    if (tmpStr.contains("END_PROGRAM") || tmpStr.contains("END_ORGANIZATION_BLOCK") ||
        tmpStr.contains("END_FUNCTION_BLOCK") || tmpStr.contains("END_FUNCTION") ||
        tmpStr.contains("@OBJECT_END") || tmpStr.contains("END_TYPE"))//if2
    { end_found = true; break; }//if2
  }//for1

  //return
  if (end_found == true)//if6
    return(i);
  else
    return(-1);
}


QString ImportExport::getCoDeSysVISU(QStringList *tmpList, int index, int end)
{ //! return CoDeSys VISU section from a list, by using a start index and an end position
	
  bool nextBlock_found=false, visu_found=false, endvisu_found=false;
  int i=0, nextBlock_pos=0, visu_pos=0, endvisu_pos=0;
  QString tmpStr="";

  for (i = index; i < end; i++)//for1
  { //get next POU block position
    tmpStr = tmpList->at(i);//get single line
    if (tmpStr.contains("PROGRAM") || tmpStr.contains("FUNCTION_BLOCK") || tmpStr.contains("FUNCTION") || tmpStr.contains("GLOBAL_VARIABLE_LIST")  || tmpStr.contains("TYPE"))//if3
    { nextBlock_found = true; nextBlock_pos = i; break; }//if1
  }//for1

  if (!nextBlock_found)//if2
    nextBlock_pos = end;
 
  for (i = index; i < nextBlock_pos; i++)//for2
  { //get data block begin position
    tmpStr = tmpList->at(i);//get single line
    if ((tmpStr.contains("VISUALISATION")) && (visu_found == false))//if3
    {  visu_found = true; visu_pos = i; }
    if ((tmpStr.contains("END_VISUALISATION")) && (endvisu_found == false))//if3
    {  endvisu_found = true; endvisu_pos = i; break; }
  }//for2

  if ((!visu_found) || (!endvisu_found))//if4
    return(""); //return if no VISU can be found
  
  tmpStr.clear(); //clear tmpStr
  for (i = visu_pos; i < endvisu_pos+1; i++)//for3
  { //collect each single line of the VISU
    tmpStr.append(tmpList->at(i));
  }//for3

  //return visu data
  return(tmpStr);
}


QString ImportExport::getSiemensDBData(QStringList *tmpList, int index, int end)
{ //! return Siemens DB data from a list, by using a start index and an end position
	
  bool nextBlock_found=false, db_found=false;
  int i=0, nextBlock_pos=0, DB_pos=0;
  QString tmpStr="";

  for (i = index; i < end; i++)//for1
  { //get next POU block position
    tmpStr = tmpList->at(i);//get single line
    if (tmpStr.contains("ORGANIZATION_BLOCK") || tmpStr.contains("FUNCTION_BLOCK") || tmpStr.contains("FUNCTION") || tmpStr.contains("TYPE"))//if3
    { nextBlock_found = true; nextBlock_pos = i; break; }//if1
  }//for1

  if (!nextBlock_found)//if2
    nextBlock_pos = end;

  for (i = index; i < nextBlock_pos; i++)//for2
  { //get data block begin position
    tmpStr = tmpList->at(i);//get single line
    if (tmpStr.contains("DATA_BLOCK"))//if3
    {  db_found = true; DB_pos = i; break;  }
  }//for2

  if (!db_found)//if4
    return(""); //return if no "DATA_BLOCK" can be found
  
  tmpStr.clear(); //clear tmpStr
  for (i = DB_pos; i < nextBlock_pos; i++)//for3
  { //collect each single line of the DB data
    tmpStr.append(tmpList->at(i));
  }//for3

  //return DB data
  return(tmpStr);
}


bool ImportExport::OpenReinforce(QStringList *tmpList, int index, int endPos, QString ExtStr)
{ //! reinforce import of file content in tmpList up from index until endPos, respecting the ExtStr value
  /*! supported extension values: */
  /*! - '.txt' */
  /*! - '.poe' */
  /*! - '.EXP' */
  /*! - '.awl' */
  /*! - '.scl' */
  /*! - '.IL' */
  /*! - '.ST' */
  
  bool      toIntDone=true, foundFT=false, flg_STsource=false, flg_ext13=false;
  int       i=0, k=0, Ext=0, startPos=0, EndOfFile=0;
  int       FileType_pos=100000, FileEnd_pos=0, END_VAR_pos=0;
  int       iffound=0, endiffound=0, semifound=0;
  QString   tmpStr="", fileName="", headerStr="", worksheetStr="", visuStr="";
  QString   FilePath="", FileType="", FncType="", FNgetter="", FileName="", FileEnd="";

  //get Ext as int for later case construct
  if (ExtStr.contains(".txt", Qt::CaseInsensitive))
    Ext = 1; //get Extension as INT for case construct
  if (ExtStr.contains(".poe", Qt::CaseInsensitive))
   Ext = 2;
  if (ExtStr.contains(".exp", Qt::CaseInsensitive))
    Ext = 3;
  if (ExtStr.contains(".awl", Qt::CaseInsensitive))
    Ext = 4;
  if (ExtStr.contains(".scl", Qt::CaseInsensitive))
    Ext = 5;
  if (ExtStr.contains(".IL", Qt::CaseInsensitive))
    Ext = 6;
  if (ExtStr.contains(".ST", Qt::CaseInsensitive))
    Ext = 7;


  //get pos information and lines content
  if (tmpList->count() < 1)//if1
    return(false);
    
  //get corrected start position, source is tmplist
  switch ( Ext )
  {
  	case 2  : { //Moeller POU  (.poe)
  	            startPos = tmpList->indexOf("$END_KM_HEADER\n", index);
  	            break; 
              } //case 2

  	case 6  : 
  	case 7  : {
                startPos = tmpList->indexOf("(*@KEY@:END_DESCRIPTION*)\n", index);
  	            break; 
  	          } //case 6, 7
  	default : { //empty
                startPos = index; break;
              } //default
  }//switch



  for (i = startPos; i < endPos; i++)//for1
  { //run through tmpList
    tmpStr = tmpList->at(i);//get single line

    //file type / function type / file name ---------------------------------------
    if ((tmpStr.contains("PROGRAM") || tmpStr.contains("ORGANIZATION_BLOCK") ||
         tmpStr.contains("FUNCTION_BLOCK") || tmpStr.contains("FUNCTION") ||
         tmpStr.contains("GLOBAL_VARIABLE_LIST") || tmpStr.contains("TYPE")) && (foundFT == false))//if2
    {
      //FileType start pos.
      FileType_pos = i;															//get file type position
      FileType = tmpStr;
      if (FileType.startsWith(" ")) //check if any blank can be found at the start
        FileType = FileType.remove(FileType.indexOf(" "), FileType.indexOf(QRegExp("[A-z]"))); //remove all blanks until a first char
      FileType = (FileType.remove(FileType.indexOf(" "), FileType.length())); 	//get File Type

      //FileType end pos. 
      FileEnd_pos = endPos;			//set end position
      FileEnd = tmpList->at(endPos);//get File End
      
      //file name handling
      fileName = tmpStr.simplified();											//get file name
      fileName = (fileName.remove(0, fileName.indexOf(" ")+1)); 				//cut file type
            
      if (tmpStr.contains("GLOBAL_VARIABLE_LIST"))//if3
      { //VAR List handling
        fileName = fileName.remove(0, fileName.indexOf("'")+1);
        fileName = fileName.remove(fileName.indexOf("'"), fileName.length());
        FileType_pos = FileType_pos+2;
        FileType = "GLOBAL_VARIABLE_LIST";
        FileEnd = ("OBJECT_END");
      }//if3
      
      if (fileName.contains(":"))
      { //get Function, marked by ':'
        FncType = fileName;
        FncType = FncType.remove(0, FncType.indexOf(":"));							//get function mark
        fileName = fileName.remove(fileName.indexOf(" "), fileName.length());	//cut function mark
      }	
      
      fileName = fileName + ExtStr; //concat filenam + extension 
      foundFT = true;       		//found flag

    }//if2
      
    //END_VAR pos. ---------------------------------------
    if (tmpStr.contains("END_VAR") || tmpStr.contains("END_STRUCT") || tmpStr.contains("(* @END_DECLARATION := '0' *)"))//if5
      END_VAR_pos = i;     //count END_VAR items
  }//for1

  //return with false if no END or file type could be found
  if((END_VAR_pos == 0) || (FileType.isEmpty()))//if2 
    return(false);



  //T E 1   S E C T I O N  ---------------------------------------
  tmpStr.clear(); //clear string
  //fill declaration section = = = = = = = = = = = =
  for (i=FileType_pos+1; i < END_VAR_pos; i++)//for1
    tmpStr.append(tmpList->at(i));
  if (!tmpList->at(END_VAR_pos).contains("(* @END_DECLARATION := '0' *)")) //check if no @END_DECL... exists
    tmpStr.append(tmpList->at(END_VAR_pos).simplified());//get last line simplified to prevent additional '\n'
  m_PageCtrl->fillCurrentDeclaration(tmpStr);
  tmpStr.clear(); //clear string



  //T E 2   S E C T I O N  ---------------------------------------
  EndOfFile = endPos; //set EOF position

  //EXTENSION Handling, source is tmplist
  switch ( Ext )
  {
  	case 1  : { //plain POE  (.txt)
  	            break;                                    //leave case contruct
              } //case 1

  	case 2  : { //Moeller POE  (.poe)
  	            QFontMetrics fm(m_PageCtrl->font);
  				m_PageCtrl->setCurrentTabStops(8*fm.width(" "));      //set TabStop width of Dec & Ins
  				
  				headerStr.clear();                          //clear MHC
        	    for (i = index; i < FileType_pos-1; i++ )   //get KM_Header in MHC
        	      headerStr.append(tmpList->value(i));      //get Strings
        	    headerStr.append("$END_KM_HEADER");         //set KMH end string (needed becc. \n would copied with org. end string)	
        	    m_PageCtrl->setCurrentKMHeader(headerStr);
        	    if (headerStr.contains("kAwl", Qt::CaseInsensitive))
        	      m_PageCtrl->setCurrentPrgLng("IL"); 		//set programming language
        	    else
        	      m_PageCtrl->setCurrentPrgLng("ST"); 		//set programming language
            	//handle Splitter Size
            	int splitSize = (tmpList->value(13).remove(0, 15)).toInt(&toIntDone, 10); //get splitSize from File
            	if (splitSize < 100)  //if size to small, set size by default value
            	  m_PageCtrl->splitSize = 200;
            	m_PageCtrl->setCurrentSplitSize(splitSize); 
            	if (tmpList->value(11).contains("kVer"))
            	  m_PageCtrl->setCurrentSplitMode("v");  	//set Splitter orientation
            	else 
            	  m_PageCtrl->setCurrentSplitMode("h");  	//set Splitter orientation
              	break;                                    	//leave case contruct
              } //case 2

  	case 3  : { //CoDeSys EXP  (.EXP)
  				QFontMetrics fm(m_PageCtrl->font);
  				m_PageCtrl->setCurrentTabStops(4*fm.width(" "));      //set TabStop width of Dec & Ins
				visuStr = getCoDeSysVISU(tmpList, endPos+1, tmpList->count());
    		    break;
              }

  	case 4  : { //Siemens AWL  (.AWL)
    	        m_PageCtrl->setCurrentPrgLng("IL");
                m_PageCtrl->setCurrentSiemensDBData(getSiemensDBData(tmpList, endPos+1, tmpList->count())); //get DB data
                break;
              } //case 4

  	case 5  : { //Siemens SCL  (.SCL)
                m_PageCtrl->setCurrentPrgLng("ST");
                m_PageCtrl->setCurrentSiemensDBData(getSiemensDBData(tmpList, endPos+1, tmpList->count())); //get DB data
                break;
              } //case 5

  	case 6  : { //KW-Software IL  (.IL)
                m_PageCtrl->setCurrentPrgLng("IL");
                
                headerStr.clear();                           //clear header string
        	    for (i = tmpList->indexOf("(*@KEY@:DESCRIPTION*)\n", index)+1; i < tmpList->indexOf("(*@KEY@:END_DESCRIPTION*)\n", index); i++ )     	//get KM_Header in MHC
        	      headerStr.append(tmpList->value(i));    	//get Strings
        	    m_PageCtrl->setCurrentKWDescription(headerStr.trimmed());
        	    
                k = tmpList->indexOf("(*@KEY@: WORKSHEET\n", END_VAR_pos)+1;
                worksheetStr = tmpList->at(k);
                worksheetStr.remove(0, worksheetStr.indexOf(" ")+1);
                m_PageCtrl->setCurrentKWWorkSheetName(worksheetStr.simplified());
	  		    END_VAR_pos = k+2;
	  		    EndOfFile = EndOfFile-1;
                break;
              } //case 6
              
    case 7  : { //KW-Software ST  (.ST)
                m_PageCtrl->setCurrentPrgLng("ST");
                
                headerStr.clear();                           //clear header string
        	    for (i = tmpList->indexOf("(*@KEY@:DESCRIPTION*)\n", index)+1; i < tmpList->indexOf("(*@KEY@:END_DESCRIPTION*)\n", index); i++ )     	//get KM_Header in MHC
        	      headerStr.append(tmpList->value(i));    	//get Strings
        	    m_PageCtrl->setCurrentKWDescription(headerStr.trimmed());
        	    
                k = tmpList->indexOf("(*@KEY@: WORKSHEET\n", END_VAR_pos)+1;
                worksheetStr = tmpList->at(k);
                worksheetStr.remove(0, worksheetStr.indexOf(" ")+1);
                m_PageCtrl->setCurrentKWWorkSheetName(worksheetStr.simplified());
	  		    END_VAR_pos = k+2;
	  		    EndOfFile = EndOfFile-1;
                break;
              } //case 7

  	default : { //empty
                break;
              } //default
  }//switch


  //fill intruction section = = = = = = = = = = = =
  for (i=END_VAR_pos+1; i < EndOfFile-1; i++)//for2
  { //get line at pos. i until i<EOF-1
    tmpStr.append(tmpList->at(i));
    if (usePrgLngDetection == true)
    { //try to get PRGLNG independent from any POU extension
      if ((tmpStr.contains(";")) && ((tmpStr.indexOf(";") < tmpStr.indexOf("(*")) || (tmpStr.indexOf("(*") < 1)))
	    semifound++;  //find indication of ST language ';'
	  if (tmpStr.contains("if ", Qt::CaseInsensitive))
	    iffound++;    //find indication of ST language 'if'
	  if (tmpStr.contains("end_if;", Qt::CaseInsensitive))
	    endiffound++; //find indication of ST language 'end_if;'
    }
  }//for2
  tmpStr.append(tmpList->at(EndOfFile-1).simplified());//get last line (at i=EOF-1) simplified to prevent additional '\n'

  
  //set programming language for current page if noPrgLngDetection is disabled
  if (usePrgLngDetection == true)
  { //m_PageCtrl->newPage() sets the predefined programming language as standard, so if 
  	//usePrgLngDetection is false, there is no need to run m_PageCtrl->setCurrentPrgLng()
  	//get some flags
    flg_STsource = (((iffound > 0) && (endiffound > 0)) || (semifound > 1));
    flg_ext13 = ((Ext == 1) || (Ext == 3));
    //revise the programming language information
    if ((flg_STsource == true) && (flg_ext13 == true ))
      m_PageCtrl->setCurrentPrgLng("ST"); 					//set if detected and extension is .txt or .exp
    if ((flg_STsource == false) && (flg_ext13 == true ))
      m_PageCtrl->setCurrentPrgLng("IL"); 					//set if detected and extension is .txt or .exp
  }

  if (Ext == 3)
    tmpStr.append("\n" + visuStr);

  //copy Str to TextEdit 2/Instruction Section & clear Str
  m_PageCtrl->fillCurrentInstruction(tmpStr);   
  tmpStr.clear();  
    
  //get file informations into page components
  m_PageCtrl->setCurrentFileEnd(FileEnd);	          	//End of File
  m_PageCtrl->setCurrentFileExt(QString::number(Ext));	//Extension of File
  m_PageCtrl->setCurrentFileListItemText(fileName);   	//add list item
  m_PageCtrl->setCurrentFileType(FileType);	          	//Type of File
  m_PageCtrl->setCurrentFncType(FncType);				//Type of Function
  m_PageCtrl->setCurrentFileName(fileName); 			//File Name
    
  //CLOSING OPERATIONS
  //delete handling for VAR Lists
  if (FileType.contains("GLOBAL_VARIABLE_LIST") || FileType.contains("TYPE"))//
	m_PageCtrl->delVARTypeItems();   //delete TE2, becc. it will not used
    
  //set Splitter visible
  m_PageCtrl->setNewSplitterVisible();
  //return with 'ok" indication
  return(true);
}




//------------------------------------------------------------------------------------
// S A V E  F U N C T I O N S  + + + + + + + + + + + + + + + + + + + + + + + + + + + +
//------------------------------------------------------------------------------------
int ImportExport::savePage(QString Str)
{ //! save page content into path which is setted in Str and call saveprepare function
	
  bool		  error = 0;
  int         Ext=0;
  QString     fileName; 

  if (Str.isEmpty())
  	return(1); //return if string is empty

  m_PageCtrl->setPageCtrlSignalsBlocked(true);  //block signals of class PageCtrl

  QStringList *tmpList = new QStringList();   
    
  //set wait cursor
  QApplication::setOverrideCursor(Qt::WaitCursor);
  emit active();
    
  //get stripped FileName from SaveDialog      
  fileName = strippedFileName(Str);                                  
  //add List item with name
  m_PageCtrl->setCurrentFileListItemText(fileName);
  //set File Name
  m_PageCtrl->setCurrentFileName(fileName);

  //get file extension
  if (fileName.contains(".txt", Qt::CaseInsensitive))
    Ext = 1; //get Extension as INT for later case construct
  if (fileName.contains(".poe", Qt::CaseInsensitive))
    Ext = 2;
  if (fileName.contains(".exp", Qt::CaseInsensitive))
    Ext = 3;
  if (fileName.contains(".awl", Qt::CaseInsensitive))
    Ext = 4;
  if (fileName.contains(".scl", Qt::CaseInsensitive))
    Ext = 5;  
  if (fileName.contains(".IL", Qt::CaseInsensitive))
    Ext = 6;
  if (fileName.contains(".ST", Qt::CaseInsensitive))
    Ext = 7;

  //execute saveprepare fcn.
  SavePrepare(m_PageCtrl->currentTabIndex(), fileName, tmpList, Ext);

  // create POU file
  QFile filePOU(Str);
  if (filePOU.open(QFile::WriteOnly | QIODevice::Text))
  { //create text stream
    QTextStream tsPOU(&filePOU);
    //set text codec
    tsPOU.setCodec(QTextCodec::codecForName("UTF-8"));
    //save tmpList from saveprepare fcn. to file
    tsPOU << tmpList->join("\n");
 
    //diverse page settings
    m_PageCtrl->setCurrentFileExt(QString::number(Ext));//Extension of FileQString::number(Ext));//Extension of File
    QString filePath = Str; //get full file name
    m_PageCtrl->setCurrentFilePath(filePath.remove(filePath.lastIndexOf("/")+1, filePath.length())); //set file path
  }
  else error = 1;

  
  if ((savePTT == true) && (m_PageCtrl->currentFileType() != "GLOBAL_VARIABLE_LIST") && (m_PageCtrl->currentFileType() != "TYPE"))
  { //change filename
  	Str = Str.remove(Str.lastIndexOf("."), Str.length()) + ".ptt";
  	//get PTT content
  	tmpList->clear();
    m_Prototype->currentPrototype(tmpList);		//get curent prototype
    // create file
    QFile filePTT(Str);
    if (filePTT.open(QFile::WriteOnly | QIODevice::Text))
    { //create text stream
      QTextStream tsPTT(&filePTT);
      //set text codec
      tsPTT.setCodec(QTextCodec::codecForName("UTF-8"));
      //save tmpList from saveprepare fcn. to file
      tsPTT << tmpList->join("\n");
    }
    else error = 2;
  }
  

  //save last dir
  if (usePresetDir == false)
    lastDir = Str.remove(Str.lastIndexOf("/")+1, Str.length()); 


  //clean up
  delete tmpList;						//delete tmpList
  
  emit modified();  					//emit modiefied information

  m_PageCtrl->setPageCtrlSignalsBlocked(false);  //unblock signals of class PageCtrl
  m_PageCtrl->setPageModifiedOf(m_PageCtrl->currentTabIndex(), false); //set page modified to false
  
  QApplication::restoreOverrideCursor();//give std. cursor back
  emit ready();							//emit ready signal
  
  return(error);						//return
}


void ImportExport::SavePrepare(int pageIndex, QString fn, QStringList *tmpList, int Ext)
{ //! prepare file storage for content at pageIndex into tmpList, with specified by filename and extension from fn and Ext 
  /*! supported extension values: */
  /*! - 0/1 = .txt */
  /*! - 2 = .poe */
  /*! - 3 = .EXP */
  /*! - 4 = .awl */
  /*! - 5 = .scl */
  /*! - 6 = .IL */
  /*! - 7 = .ST */
  
  int       i=0;
  QString   FileType, FncType, FileName, FileEnd, TimeString; //File name & TimeStamp getter
  QString   tmpStr, CoDeSysV2_tag1="", CoDeSysV2_tag2="", S7_tag1="", S7_tag2="", KW_tag1="", KW_tag2="", KW_tag3="";

  //prepare file information from Page
  FileType = m_PageCtrl->fileTypeOf(pageIndex);	//Type of File
  FncType  = m_PageCtrl->fncTypeOf(pageIndex);	//Type of Function
  FileEnd  = m_PageCtrl->fileEndOf(pageIndex);	//End of File
  
  //fileName String
  if (fn.contains("."))
    FileName = fn.remove(fn.lastIndexOf("."), fn.length()); //extraction for save in file
  else 
  	FileName = fn;
	
  //EXT selection
  switch ( Ext )
  {
  	case 1  : { //plain POE (.txt)
  	            if (FileType == "ORGANIZATION_BLOCK")
                  FileType = "PROGRAM";
                if (FileEnd == "END_ORGANIZATION_BLOCK")
                  FileEnd = "END_PROGRAM";
  	            
  	            break;                                    	//leave case contruct
              } //case 1
              
  	case 2  : { //Moeller POE (.poe)
  	            tmpList->append(createKMHeader(m_PageCtrl->KMHeaderOf(pageIndex))); //copy Moeller header to stringlist
                
                break;                                    	//leave case contruct
              } //case 2

  	case 3  : { //CoDeSys EXP (.EXP)
  	            //normal handling for OBs
                if (FileType == "ORGANIZATION_BLOCK")
                  FileType = "PROGRAM";
                if (FileEnd == "END_ORGANIZATION_BLOCK")
                  FileEnd = "END_PROGRAM";

                //set first line as empty in stringlist
                tmpList->append(" ");

                //handling for VAR LIST files
                if (FileType == "GLOBAL_VARIABLE_LIST")
                  tmpList->append("(* @GLOBAL_VARIABLE_LIST := '" + FileName+ "' *)");
                else
                //handling for P, FB, F files
                tmpList->append(" ");

                //get path info
                tmpList->append("(* @PATH := '' *)");

                //get symfileflags
                if (FileType == "PROGRAM" || FileType == "GLOBAL_VARIABLE_LIST")
                  tmpList->append("(* @SYMFILEFLAGS := '2048' *)");
                else
                  if (FileType != "TYPE")
  	                tmpList->append("(* @SYMFILEFLAGS := '0' *)");
  	              
  	            CoDeSysV2_tag1 = "(* @END_DECLARATION := '0' *)";
  	            CoDeSysV2_tag2.append("(* @OBJECT_END := '" + FileName + "' *)\n");
                CoDeSysV2_tag2.append("(* @CONNECTIONS := " + FileName + "\n");
                CoDeSysV2_tag2.append("FILENAME : ''\n");
                CoDeSysV2_tag2.append("FILETIME : 0\n");
                CoDeSysV2_tag2.append("EXPORT : 0\n");
                CoDeSysV2_tag2.append("NUMOFCONNECTIONS : 0\n");
                CoDeSysV2_tag2.append("*)");
  	            
  	            break;                                    	//leave case contruct
              } //case 3

  	case 4  : { //Siemens AWL (.AWL)
  	            if (FileType == "PROGRAM")
                  FileType = "ORGANIZATION_BLOCK";
                if (FileEnd == "END_PROGRAM")
                  FileEnd = "END_ORGANIZATION_BLOCK";
                
                S7_tag1 = "TITLE =\n//Block comment\nAUTHOR : \nFAMILY : \nNAME : \nVERSION : 1.0";
                S7_tag2 = "BEGIN";
                  
                m_PageCtrl->setPrgLngOf(pageIndex, "IL");	//programming language
  	            
  	            break;                                    	//leave case contruct
              } //case 4

  	case 5  : { //Siemens SCL (.SCL)
  	            if (FileType == "PROGRAM")
                  FileType = "ORGANIZATION_BLOCK";
                if (FileEnd == "END_PROGRAM")
                  FileEnd = "END_ORGANIZATION_BLOCK";
                
                S7_tag1 = "TITLE =\n//Block comment\nAUTHOR : \nFAMILY : \nNAME : \nVERSION : 1.0";
                S7_tag2 = "BEGIN";
                  
                m_PageCtrl->setPrgLngOf(pageIndex, "ST");	//programming language
               
  	            break;                                    	//leave case contruct
              } //case 5

  	case 6 : { //KW-Software IL (.IL)
                if (FileType == "ORGANIZATION_BLOCK")
                  FileType = "PROGRAM";
                if (FileEnd == "END_ORGANIZATION_BLOCK")
                  FileEnd = "END_PROGRAM";

                m_PageCtrl->setPrgLngOf(pageIndex, "IL");	//programming language
                tmpList->append("(*@PROPERTIES_EX@");
				tmpStr = m_PageCtrl->KWPropertiesOf(pageIndex);
				tmpStr.replace(QString("IEC_LANGUAGE: ST"), QString("IEC_LANGUAGE: IL"));
				m_PageCtrl->setKWPropertiesOf(pageIndex, tmpStr);
                tmpList->append(m_PageCtrl->KWPropertiesOf(pageIndex));
                tmpList->append("*)");
                tmpList->append("(*@KEY@:DESCRIPTION*)");
                tmpList->append(m_PageCtrl->KWDescriptionOf(pageIndex));
                tmpList->append("(*@KEY@:END_DESCRIPTION*)");
                
                //KW_tag1 = "\n(*Group:Default*)\n\n";
                KW_tag2 = ("\n\n(*@KEY@: WORKSHEET\nNAME: " + FileName + "\nIEC_LANGUAGE: IL\n*)");
                KW_tag3 = "(*@KEY@: END_WORKSHEET *)";
                
  	            break;                                    	//leave case contruct
              } //case 6

  	case 7  : { //KW-Software ST (.ST)
                if (FileType == "ORGANIZATION_BLOCK")
                  FileType = "PROGRAM";
                if (FileEnd == "END_ORGANIZATION_BLOCK")
                  FileEnd = "END_PROGRAM";

                m_PageCtrl->setPrgLngOf(pageIndex, "ST");	//programming language
                tmpList->append("(*@PROPERTIES_EX@");
				tmpStr = m_PageCtrl->KWPropertiesOf(pageIndex);
				tmpStr.replace(QString("IEC_LANGUAGE: IL"), QString("IEC_LANGUAGE: ST"));
				m_PageCtrl->setKWPropertiesOf(pageIndex, tmpStr);
                tmpList->append(m_PageCtrl->KWPropertiesOf(pageIndex));
                tmpList->append("*)");
                //tmpList->append("(*@PROPERTIES_EX@\nTYPE: POU\nLOCALE: 0\nIEC_LANGUAGE: ST\nPLC_TYPE: independent\nPROC_TYPE: independent\n*)");
                tmpList->append("(*@KEY@:DESCRIPTION*)");
                tmpList->append(m_PageCtrl->KWDescriptionOf(pageIndex));
                tmpList->append("(*@KEY@:END_DESCRIPTION*)");
                
                //KW_tag1 = "\n(*Group:Default*)\n\n";
                KW_tag2 = ("\n\n(*@KEY@: WORKSHEET\nNAME: " + FileName + "\nIEC_LANGUAGE: ST\n*)");
                KW_tag3 = "(*@KEY@: END_WORKSHEET *)";
                
  	            break;                                    	//leave case contruct
              } //case 7

    default : { //empty
                if (FileType == "ORGANIZATION_BLOCK")
                  FileType = "PROGRAM";
                if (FileEnd == "END_ORGANIZATION_BLOCK")
                  FileEnd = "END_PROGRAM";
  	            
  	            break;
              } //default
  }//switch


  //different terms for manufacturer compliances
  //get Type + Name line in strinlist
  if ((FileType != "GLOBAL_VARIABLE_LIST"))
    tmpList->append(FileType + " " + FileName + " " + FncType);
  
  //get spec. lines for KW-Software, not needed
  //if ((Ext == 6) || (Ext == 7))
  //  tmpList->append(KW_tag1);  

  //g e t   T E 1   i n   s t r i n g l i s t - - - 
  tmpList->append(m_PageCtrl->declarationOf(pageIndex)); //former function: .split("\n", QString::KeepEmptyParts));

  //insert tags for CoDeSys between TE1 & TE2  
  if ((Ext == 3) && (FileType != "GLOBAL_VARIABLE_LIST") && (FileType != "TYPE"))
    tmpList->append(CoDeSysV2_tag1);
    
  //insert tag1 for Siemens Step7 between TE1 & TE2 if no 'BEGIN' can be found
  if ((FileType != "GLOBAL_VARIABLE_LIST") && (FileType != "TYPE"))
    if (((Ext == 4) || (Ext == 5)) && (!m_PageCtrl->instructionOf(pageIndex).contains("BEGIN",Qt::CaseSensitive)))
      tmpList->append(S7_tag2);
  
  //insert tags for KW-Soft betweenT E1 & TE2
  if ((Ext == 6) || (Ext == 7))
    tmpList->append(KW_tag2);

  //g e t   T E 2   i n   s t r i n g l i s t  if file not VAR LIST - - - 
  tmpStr.clear();
  if ((FileType != "GLOBAL_VARIABLE_LIST") && (FileType != "TYPE"))
    tmpStr = m_PageCtrl->instructionOf(pageIndex);
    
  //get FileEnd for EXP files  (EXT 3)
  if ((Ext == 3) && (FileType != "GLOBAL_VARIABLE_LIST") && (FileType != "TYPE"))
  { //handling to place file end correct if the file contains VISU information
    if (tmpStr.contains("VISUALISATION"))
    { //if VISU found get pos and set FileEnd mark + \n   
      i = tmpStr.indexOf("VISUALISATION");
      tmpStr.insert(i, (FileEnd + "\n(* @PATH := '' *)\n\n"));
    }
    else //if no VISU found add FileEnd mark
      tmpStr.append("\n" + FileEnd); 
  }
  //get FileEnd for EXP files which are TYPE (EXT 3)
  if ((Ext == 3) && (FileType == "TYPE"))
    tmpStr.append(FileEnd);
    
  tmpList->append(tmpStr);  //get TE2 into string list

  //insert tags for KW-Soft behind instructions
  if ((Ext == 6) || (Ext == 7))
    tmpList->append(KW_tag3);
    
  //get FileEnd in stringlist if not EXT 3
  if (Ext != 3)
      tmpList->append(FileEnd);
  
  //get Siemens DB Data after file end mark in stringlist
  if ((Ext == 4) || (Ext == 5))
    tmpList->append(m_PageCtrl->SiemensDBDataOf(pageIndex));
   
  //FileEnd lines for VAR LIST
  if ((Ext == 3) && (FileType == "GLOBAL_VARIABLE_LIST"))
    tmpList->append(CoDeSysV2_tag2);

  //FileEnd lines for TYPE
  if ((Ext == 3) && (FileType == "TYPE"))
    tmpList->append("\n\n" + CoDeSysV2_tag1);
    	
}




//=============================================================================
// - - - diverse private functions - - - 
QString ImportExport::strippedFileName(const QString &fullFileName)
{ //! strip filename from fullFileName (path + file name)
  
  return(QFileInfo(fullFileName).fileName().simplified());
}
